// PROGRAM na programovani PROM MH74188 (32x 8 bitu)
// pomoci specielniho pripravku a karty s UART 8255 a casovace 8253
program prgogramator74188;

uses	go32, crt, numbers, fpc024;

// RIDICI PORT PSx
//      0       PORT C lower    1 input / 0 output
//      1       PORT B          1 input / 0 output
//      2       MODE            1 mode 1 / 0 mode 0
//      3       PORT C upper    1 input / 0 output
//      4       PORT A          1 input / 0 output
//      5       MODE            00 mode 0 / 01 mode 1 / 1x mode 2
//      6       MODE            ---- // ----
//      7       MODE FLAG       1 active

var     Data: array [0..31] of byte;

procedure WaitToKey;
begin
	repeat until keypressed; readkey;
end;

function ReadByte (address: byte): byte;        // precte byte z PROM
begin
        address := address and %00011111;// adresa musi byt v rozsahu 0 - 31
        outportb (PS1, %10010000);       // A vstup, ostatni vystupy
        outportb (PC1, 0);
        outportb (PB1, address);
	delay (1);
        ReadByte := inportb (PA1);
end;

procedure DumpHex;      // zobrazi obsah promenne Data
var     m,n: integer;
begin
        for n:=0 to 7 do
        begin
                write ('[', DecToHex (n*4),']  ');
                for m:=0 to 3 do write (DecToHex (Data [n*4+m]),', ');
                writeln;
        end;
end;

procedure DumpBin;      // zobrazi obsah promenne Data
var     m,n: integer;
begin
        for n:=0 to 7 do
        begin
                write ('[', DecToBin (n*4),']  ');
                for m:=0 to 3 do write (DecToBin (Data [n*4+m]),', ');
                writeln;
        end;
end;

procedure ReadPROM;     // nacte obsah PROM do promenne Data
var     n: integer;
begin
        for n:=0 to 31 do Data [n] := ReadByte (n);
end;

function WriteBitToPROM (address: byte; value: byte; mask: byte): boolean;
var	error: boolean;	// bude tru pokud se nepovede naprogramovat bit
	done: boolean;	// pomocna promenna pro programovaci cyklus
	progtime: byte;	// programovaci cas v ms
	b: byte;	// pomocna promenna pro verifikaci
begin
	// NUTNY PREDPOKLAD:
	// po dobu programovani jednoho bitu / nesmi jich byt vice najednou
	// potece do programovaneho vstupu az 150mA a do napajeciho az 450mA
	// nutno zajistit, aby byl v log 0 nastaven jen programovaci bit ???

	// nastavit adresu a odselektovat
	address := address and %00011111;
	outportb (PB1, address);
	outportb (PC1, %00000001);
	delay (1);
	error := false;
	// pokud je bit k naprogamovani v log 1 pak budu programovat
	if ((value and mask) > 0) then
	begin
		progtime := 4;	// 4ms pocatecni programovaci cas
		done := false;
		// VERIFY - je nutno programovat?
		outportb (PC1, %00000000);
		delay (1);
		b := inportb (PA1);
		if ((b and mask) > 0) then
		begin
			error := false;
		end else begin
			repeat
				// PROGRAMUJ
				// nastavit programovaci napeti, ale zatim neselektovat
				outportb (PC1, %00000011);
				delay (1);
				// nastavit selekt a drzet programovaci napeti po urceny cas
				outportb (PC1, %00000010);
				delay (progtime); // typicky prg. cas je 1ms, max. 20ms
				// shodit select
				outportb (PC1, %00000011);
				delay (1);
				// shodit programovaci napeti
				outportb (PC1, %00000001);
				delay (100);	// zotaveni pro rele ...
				// VERIFY
				// vyselektovat bez nastaveni programovaciho napeti
				outportb (PC1, %00000000);
				delay (1);
				b := inportb (PA1);
				// odselektovat
				outportb (PC1, %00000001);
				// a vyhodnotit vysledek verifikace
				if ((b and mask) = 0) then
					// v pripade neuspechu bude program.
					// cas prodlouzen o 4ms az do 20ms	
					progtime := progtime + 4
				else begin
					done := true;
					error := false;
				end;
				if (progtime > 20) then
				begin
					done := true;
					error := true;
				end;
			until done;
		end;
	end;
	WriteBitToPROM := error;
end;

function VerifyByte (address: byte ): boolean;
begin
	if (Data [address] = ReadByte (address)) then VerifyByte := true else VerifyByte := false;
end;

procedure VerifyPROM;
var	n: integer;
	failed: boolean;
begin
	failed := false;
	for n:=0 to 31 do
	begin
		if (not (VerifyByte (n))) then
		begin
			write ('[',DecToHex (n),'] ');
			textcolor (12);
			write (' incorrect! ');
			textcolor (7);
			write ('DATA = ',DecToBin (Data [n]),'  ');
			write ('PROM = ',DecToBin (ReadByte (n)));
			writeln;
			failed := true;
		end;
	end;
	if (not failed) then
	begin
		textcolor (10);
		write ('Data succesfully verified.');
		textcolor (7);
		writeln;
	end;
end;

procedure WritePROM;    // zapise obsah promenne Data do PROM
var     bitnumber: byte;
	mask, a, bit: byte;
	result: boolean;
begin
	mask := 1;
	for bit:=0 to 7 do
        begin
		textcolor (12); write ('Bit ',bit,' (',DecToBin (mask),') ');
		textcolor (14); write ('Press any key.     ');
		WaitToKey;
		for a:=0 to 31 do
		begin
//			write ('[',DecToHex(a and mask),'] ', DecToHex (Data [a and mask]),'   ');
			result := WriteBitToPROM (a, Data [a], mask);
			if ((Data[a] and mask) > 0) then
			begin
				textcolor (12);
				if (result) then 
					write ('?')
				else
					write ('1');
			end else begin
				textcolor (10);
				write ('0');
			end;
		end;
		textcolor (7); writeln;
		mask := mask * 2;
        end;
end;

procedure SaveData (filename: string);  // ulozi obsah promenne Data do souboru
var     n: integer;
        f: file of byte;
begin
        assign (f, filename);
        rewrite (f);
        for n:=0 to 31 do write (f, Data [n]);
        close (f);
	writeln ('Data saved to file ',filename,'.');
end;

procedure LoadData (filename: string);  // nacte obsah souboru do promenne Data
var     n: integer;
        f: file of byte;
begin
        for n:=0 to 31 do Data [n] := %01010101;
        assign (f, filename);
        reset (f);
        for n:=0 to 31 do read (f, Data [n]);
        close (f);
	writeln ('Data loaded from file ',filename,'.');
end;

procedure Menu;
var	key: char;
	done: boolean;
begin
	done := false;
	repeat
		ClrScr;
		
		textcolor (15);
		writeln ('L - Load data from file');
		writeln ('S - Save data to file');
		writeln ('P - Write data to PROM');
		writeln ('V - Verify data in PROM');
		writeln ('R - Read data from PROM');
		writeln ('B - Binary dump');
		writeln ('H - Hexa dump');
		writeln ('ECS - Quit');

		repeat until keypressed;
		key := readkey;
		case key of
			'l': LoadData ('output.rom');
			's': SaveData ('output.rom');
			'p': WritePROM;
			'v': VerifyPROM;
			'r': ReadPROM;
			'b': DumpBin;
			'h': DumpHex;
			char (27): done := true;
			else
			begin
				textcolor (4);
				writeln ('Wrong key! Be carefull.');
			end;
		end;
		
		textcolor (14); writeln ('Press any key.');
		WaitToKey;

	until done;
end;

var     b: byte;
        n: integer;
begin
        // inicializace portu
        outportb (PS1, %10010000);	// A vstup, ostatni vystupy
	outportb (PC1, 0);		// programovani NE = 0, /CS aktivni v 0
	outportb (PB1, 0);		// adresa 0
	// hlavni menu programu
	Menu;
	textcolor (7); writeln ('O.K');
end.
